@@ -21,8 +21,10 @@ mc.crisk.bart <- function(
     x.train = matrix(0.0, 0L, 0L), y.train=NULL,
     x.train2 = x.train, y.train2=NULL,
     times=NULL, delta=NULL,
-    x.test = matrix(0.0, 0L, 0L), x.test2 = x.test,
-    cond=NULL,
+    x.test = matrix(0.0, 0L, 0L), x.test2 = x.test, cond=NULL,
+    sparse=FALSE, a=0.5, b=1, augment=FALSE, rho=NULL, rho2=NULL,
+    xinfo=matrix(0.0,0,0), xinfo2=matrix(0.0,0,0), usequants=FALSE,
+    cont=FALSE, rm.const=TRUE, type='pbart',
     k = 2.0, ## BEWARE: do NOT use k for other purposes below
     power = 2.0, base = 0.95,
     binaryOffset = NULL,
@@ -31,9 +33,11 @@ mc.crisk.bart <- function(
     ndpost = 1000L, nskip = 250L,
     keepevery = 10L,
     nkeeptrain=ndpost, nkeeptest=ndpost,
-    nkeeptestmean=ndpost, nkeeptreedraws=ndpost,
+    ##nkeeptestmean=ndpost,
+    nkeeptreedraws=ndpost,
     printevery=100L,
-    treesaslists=FALSE, keeptrainfits=TRUE,
+    ##treesaslists=FALSE,
+    keeptrainfits=TRUE,
     id=NULL,    ## crisk.bart only
     seed = 99L, mc.cores = 2L, nice=19L
 )
@@ -45,6 +49,11 @@ mc.crisk.bart <- function(
     set.seed(seed)
     parallel::mc.reset.stream()
 
+    x.train2 <- bartModelMatrix(x.train2)
+    x.test2 <- bartModelMatrix(x.test2)
+    x.train <- bartModelMatrix(x.train)
+    x.test <- bartModelMatrix(x.test)
+
     if(length(y.train)==0) {
         pre <- crisk.pre.bart(times, delta, x.train, x.test, x.train2, x.test2)
 
@@ -110,17 +119,22 @@ mc.crisk.bart <- function(
         parallel::mcparallel({psnice(value=nice);
               crisk.bart(x.train=x.train, y.train=y.train,
                          x.train2=x.train2, y.train2=y.train2,
-                         x.test=x.test, x.test2=x.test2,
-                         cond=cond, k=k, power=power, base=base,
+                         x.test=x.test, x.test2=x.test2, cond=cond,
+                         sparse=sparse, a=a, b=b, augment=augment, rho=rho, rho2=rho2,
+                         xinfo=xinfo, xinfo2=xinfo2, usequants=usequants,
+                         cont=cont, rm.const=rm.const, type=type,
+                         k=k, power=power, base=base,
                          binaryOffset=binaryOffset,
                          binaryOffset2=binaryOffset2,
                          ntree=ntree, numcut=numcut,
                          ndpost=mc.ndpost, nskip=nskip,
                          keepevery = keepevery,
                          nkeeptrain=mc.ndpost, nkeeptest=mc.ndpost,
-                         nkeeptestmean=mc.ndpost, nkeeptreedraws=mc.ndpost,
+                         ##nkeeptestmean=mc.ndpost,
+                         nkeeptreedraws=mc.ndpost,
                          printevery=printevery,
-                         treesaslists=FALSE, keeptrainfits=TRUE)},
+                         ##treesaslists=FALSE,
+                         keeptrainfits=TRUE)},
               silent=(i!=1))
               ## to avoid duplication of output
               ## capture stdout from first posterior only
@@ -135,7 +149,7 @@ mc.crisk.bart <- function(
             if(h==1 & i==mc.cores) {
                 post <- post.list[[1]][[mc.cores]]
 
-                p <- ncol(x.train)
+                p <- ncol(x.train[ , post$rm.const])
                 old.text <- paste0(as.character(mc.ndpost), ' ', as.character(ntree), ' ', as.character(p))
                 ##old.text <- paste0(as.character(mc.nkeep), ' ', as.character(ntree), ' ', as.character(p))
                 old.stop <- nchar(old.text)
@@ -146,7 +160,7 @@ mc.crisk.bart <- function(
                                                    as.character(p)),
                                             post$treedraws$trees)
 
-                p <- ncol(x.train2)
+                p <- ncol(x.train2[ , post$rm.const2])
                 old.text <- paste0(as.character(mc.ndpost), ' ', as.character(ntree), ' ', as.character(p))
                 ##old.text <- paste0(as.character(mc.nkeep), ' ', as.character(ntree), ' ', as.character(p))
                 old.stop2 <- nchar(old.text)
@@ -158,29 +172,20 @@ mc.crisk.bart <- function(
                                             post$treedraws2$trees)
             }
             else {
-                if(length(post$yhat.test)>0)
+                if(length(x.test)>0) {
                     post$yhat.test <- rbind(post$yhat.test, post.list[[h]][[i]]$yhat.test)
-
-                if(length(post$yhat.test2)>0)
                     post$yhat.test2 <- rbind(post$yhat.test2, post.list[[h]][[i]]$yhat.test2)
-
-                if(length(post$prob.test)>0)
                     post$prob.test <- rbind(post$prob.test, post.list[[h]][[i]]$prob.test)
-
-                if(length(post$prob.test2)>0)
                     post$prob.test2 <- rbind(post$prob.test2, post.list[[h]][[i]]$prob.test2)
-
-                if(length(post$cif.test)>0)
                     post$cif.test <- rbind(post$cif.test, post.list[[h]][[i]]$cif.test)
-
-                if(length(post$cif.test2)>0)
                     post$cif.test2 <- rbind(post$cif.test2, post.list[[h]][[i]]$cif.test2)
-
-                if(length(post$surv.test)>0)
                     post$surv.test <- rbind(post$surv.test, post.list[[h]][[i]]$surv.test)
+                }
 
                 post$varcount <- rbind(post$varcount, post.list[[h]][[i]]$varcount)
                 post$varcount2 <- rbind(post$varcount2, post.list[[h]][[i]]$varcount2)
+                post$varprob <- rbind(post$varprob, post.list[[h]][[i]]$varprob)
+                post$varprob2 <- rbind(post$varprob2, post.list[[h]][[i]]$varprob2)
 
                 post$treedraws$trees <- paste0(post$treedraws$trees,
                                                substr(post.list[[h]][[i]]$treedraws$trees, old.stop+2,
@@ -190,26 +195,30 @@ mc.crisk.bart <- function(
                                                substr(post.list[[h]][[i]]$treedraws2$trees, old.stop2+2,
                                                       nchar(post.list[[h]][[i]]$treedraws2$trees)))
 
-                if(treesaslists) {
-                    post$treedraws$lists <-
-                        c(post$treedraws$lists, post.list[[h]][[i]]$treedraws$lists)
+                ## if(treesaslists) {
+                ##     post$treedraws$lists <-
+                ##         c(post$treedraws$lists, post.list[[h]][[i]]$treedraws$lists)
 
-                    post$treedraws2$lists <-
-                        c(post$treedraws2$lists, post.list[[h]][[i]]$treedraws2$lists)
-                }
+                ##     post$treedraws2$lists <-
+                ##         c(post$treedraws2$lists, post.list[[h]][[i]]$treedraws2$lists)
+                ## }
             }
 
             post.list[[h]][[i]] <- NULL
             }
 
-        if(length(post$cif.test.mean)>0)
+        if(length(x.test)>0) {
+            post$prob.test.mean <- apply(post$prob.test, 2, mean)
+            post$prob.test2.mean <- apply(post$prob.test2, 2, mean)
             post$cif.test.mean <- apply(post$cif.test, 2, mean)
-
-        if(length(post$cif.test2.mean)>0)
             post$cif.test2.mean <- apply(post$cif.test2, 2, mean)
-
-        if(length(post$surv.test.mean)>0)
             post$surv.test.mean <- apply(post$surv.test, 2, mean)
+        }
+        
+        post$varcount.mean <- apply(post$varcount, 2, mean)
+        post$varcount2.mean <- apply(post$varcount2, 2, mean)
+        post$varprob.mean <- apply(post$varprob, 2, mean)
+        post$varprob2.mean <- apply(post$varprob2, 2, mean)
 
         attr(post, 'class') <- 'criskbart'
 
