@@ -21,16 +21,21 @@ mc.recur.bart <- function(
     y.train=NULL, times=NULL, delta=NULL,
     x.test = matrix(0.0, 0L, 0L),
     x.test.nogrid = FALSE, ## you may not need the whole grid
+    sparse=FALSE, a=0.5, b=1, augment=FALSE, rho=NULL,
+    xinfo=matrix(0.0,0,0), usequants=FALSE,
+    cont=FALSE, rm.const=TRUE, type='pbart',
     k = 2.0, ## BEWARE: do NOT use k for other purposes below
     power = 2.0, base = 0.95,
-    binaryOffset = NULL, ##M=1,
+    binaryOffset = NULL,
     ntree = 50L, numcut = 100L,
     ndpost = 1000L, nskip = 250L,
     keepevery = 10L,
     nkeeptrain=ndpost, nkeeptest=ndpost,
-    nkeeptestmean=ndpost, nkeeptreedraws=ndpost,
+    ##nkeeptestmean=ndpost,
+    nkeeptreedraws=ndpost,
     printevery=100L,
-    treesaslists=FALSE, keeptrainfits=TRUE,
+    ##treesaslists=FALSE,
+    keeptrainfits=TRUE,
     seed = 99L,    ## only used by mc.recur.bart
     mc.cores = 2L, ## ditto
     nice=19L       ## ditto
@@ -43,6 +48,9 @@ mc.recur.bart <- function(
     set.seed(seed)
     parallel::mc.reset.stream()
 
+    x.train <- bartModelMatrix(x.train)
+    x.test <- bartModelMatrix(x.test)
+
     if(length(y.train)==0) {
         recur <- recur.pre.bart(times, delta, x.train, x.test)
 
@@ -97,13 +105,18 @@ mc.recur.bart <- function(
             parallel::mcparallel({psnice(value=nice);
                 recur.bart(x.train=x.train, y.train=y.train,
                            x.test=x.test, x.test.nogrid=x.test.nogrid,
+                           sparse=sparse, a=a, b=b, augment=augment, rho=rho,
+                           xinfo=xinfo, usequants=usequants,
+                           cont=cont, rm.const=rm.const, type=type,
                            k=k, power=power, base=base,
-                           binaryOffset=binaryOffset, ##M=M,
+                           binaryOffset=binaryOffset,
                            ntree=ntree, numcut=numcut,
-                           ndpost=mc.ndpost, nskip=nskip,
+                           ndpost=mc.ndpost, nskip=nskip, keepevery=keepevery,
                            nkeeptrain=mc.ndpost, nkeeptest=mc.ndpost,
-                           nkeeptestmean=mc.ndpost, nkeeptreedraws=mc.ndpost,
-                           printevery=printevery, treesaslists=treesaslists)},
+                           ##nkeeptestmean=mc.ndpost,
+                           nkeeptreedraws=mc.ndpost,
+                           printevery=printevery, ##treesaslists=treesaslists,
+                           keeptrainfits=keeptrainfits)},
                 silent=(i!=1))
             ## to avoid duplication of output
             ## capture stdout from first posterior only
@@ -118,7 +131,7 @@ mc.recur.bart <- function(
             if(h==1 & i==mc.cores) {
                 post <- post.list[[1]][[mc.cores]]
 
-                p <- ncol(x.train)
+                p <- ncol(x.train[ , post$rm.const])
 
                 old.text <- paste0(as.character(mc.ndpost), ' ', as.character(ntree), ' ', as.character(p))
                 ##old.text <- paste0(as.character(mc.nkeep), ' ', as.character(ntree), ' ', as.character(p))
@@ -131,49 +144,55 @@ mc.recur.bart <- function(
                                             post$treedraws$trees)
             }
             else {
-                post$yhat.train <- rbind(post$yhat.train, post.list[[h]][[i]]$yhat.train)
+                if(keeptrainfits) {
+                    post$yhat.train <- rbind(post$yhat.train, post.list[[h]][[i]]$yhat.train)
+                    post$prob.train <- rbind(post$prob.train, post.list[[h]][[i]]$prob.train)
+                    post$haz.train <- rbind(post$haz.train, post.list[[h]][[i]]$haz.train)
+                    post$cum.train <- rbind(post$cum.train, post.list[[h]][[i]]$cum.train)
+                }
 
-                ## if(length(post$cum.train)>0) {
-                ##     post$cum.train <- rbind(post$cum.train, post.list[[h]][[i]]$cum.train)
-                ##     post$haz.train <- rbind(post$haz.train, post.list[[h]][[i]]$haz.train)
-                ## }
-
-                if(length(post$yhat.test)>0) {
+                if(length(x.test)>0) {
                     post$yhat.test <- rbind(post$yhat.test, post.list[[h]][[i]]$yhat.test)
+                    post$prob.test <- rbind(post$prob.test, post.list[[h]][[i]]$prob.test)
                     post$haz.test <- rbind(post$haz.test, post.list[[h]][[i]]$haz.test)
                     if(!x.test.nogrid) post$cum.test <- rbind(post$cum.test, post.list[[h]][[i]]$cum.test)
                 }
 
-                if(length(post$sigma)>0)
-                    post$sigma <- c(post$sigma, post.list[[h]][[i]]$sigma)
+                ## if(length(post$sigma)>0)
+                ##     post$sigma <- c(post$sigma, post.list[[h]][[i]]$sigma)
 
                 post$varcount <- rbind(post$varcount, post.list[[h]][[i]]$varcount)
+                post$varprob <- rbind(post$varprob, post.list[[h]][[i]]$varprob)
 
                 post$treedraws$trees <- paste0(post$treedraws$trees,
                                                substr(post.list[[h]][[i]]$treedraws$trees, old.stop+2,
                                                       nchar(post.list[[h]][[i]]$treedraws$trees)))
 
-                if(treesaslists) post$treedraws$lists <-
-                                     c(post$treedraws$lists, post.list[[h]][[i]]$treedraws$lists)
+                ## if(treesaslists) post$treedraws$lists <-
+                ##                      c(post$treedraws$lists, post.list[[h]][[i]]$treedraws$lists)
 
             }
 
             post.list[[h]][[i]] <- NULL
         }
 
-        post$yhat.train.mean <- apply(post$yhat.train, 2, mean)
+        ##post$yhat.train.mean <- apply(post$yhat.train, 2, mean)
 
-        ## if(length(post$cum.train)>0) {
-        ##     post$cum.train.mean <- apply(post$cum.train, 2, mean)
-        ##     post$haz.train.mean <- apply(post$haz.train, 2, mean)
-        ## }
+        if(keeptrainfits) {
+            post$prob.train.mean <- apply(post$prob.train, 2, mean)
+            post$haz.train.mean <- apply(post$haz.train, 2, mean)
+            post$cum.train.mean <- apply(post$cum.train, 2, mean)
+        }
 
-        if(length(post$yhat.test)>0) {
-            post$yhat.test.mean <- apply(post$yhat.test, 2, mean)
+        if(length(x.test)>0) {
+            post$prob.test.mean <- apply(post$prob.test, 2, mean)
             post$haz.test.mean <- apply(post$haz.test, 2, mean)
             if(!x.test.nogrid) post$cum.test.mean <- apply(post$cum.test, 2, mean)
         }
 
+        post$varcount.mean <- apply(post$varcount, 2, mean)
+        post$varprob.mean <- apply(post$varprob, 2, mean)
+
         attr(post, 'class') <- 'recurbart'
 
         return(post)
